package com.example.duonachi.activity;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.icu.text.SimpleDateFormat;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;

import com.example.duonachi.R;
import com.example.duonachi.Utils.Accessapi;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class Identify extends AppCompatActivity {

    private Button BackButton;
    private Button PhotoIdentifyButton;
    private Button TakePhotoButton;
    private Button StartIdentifyButton;
    private Uri ImageUri;//拍摄照片的URI
    private String tempFilePath;
    private ImageView figure_ready_imageview;
    private static final int REQUEST_CODE_SELECT_PHOTO = 0;//相册请求码
    private static final int TAKE_PHOTO = 1;//相机请求码
    private static final int NUM_THREADS = 4;  // 线程池大小，可以根据需要调整

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_identify);

        figure_ready_imageview = findViewById(R.id.figure_ready_imageview);

        BackButton = findViewById(R.id.BackButton);
        BackButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(Identify.this, Mainpage .class);
                startActivity(intent);
            }
        });

        //相册识别按钮
        PhotoIdentifyButton = findViewById(R.id.PhotoIdentifyButton);
        PhotoIdentifyButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                // 创建意图，启动相册应用
                Intent intent = new Intent();
                // 设置动作和类别，确保打开的是相册应用而不是其他应用
                intent.setAction(Intent.ACTION_GET_CONTENT);
                intent.addCategory(Intent.CATEGORY_OPENABLE);
                // 设置类型为图片，确保打开的是图片选择器而不是其他类型的文件选择器
                intent.setType("image/*");
                //使用自定义的请求码，以便在onActivityResult中识别请求
                startActivityForResult(intent, REQUEST_CODE_SELECT_PHOTO);
            }
        });

        //开始识别按钮
        StartIdentifyButton = findViewById(R.id.StartIdentifyButton);
        StartIdentifyButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //调用线程异步处理网络请求
                ExecutorService executorService = Executors.newFixedThreadPool(NUM_THREADS);
                executorService.submit(() -> {
                    try {
                        String idt_result = Accessapi.dish(tempFilePath);
                        Log.d("idt_result",idt_result);
                        Intent intent = new Intent(Identify.this, IdentifyResult.class);
                        intent.putExtra("result",idt_result);
                        if (ImageUri == null)
                            Toast.makeText(Identify.this, "未上传新照片", Toast.LENGTH_SHORT).show();
                        else{
                            String Imageuri = ImageUri.toString();
                            intent.putExtra("figureUri",Imageuri);
                            startActivity(intent);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        executorService.shutdown();  // 记得关闭线程池
                    }
                });
            }
        });

        //拍照上传按钮
        TakePhotoButton = findViewById(R.id.TakePhotoButton);
        TakePhotoButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //动态申请权限
                if (ContextCompat.checkSelfPermission(Identify.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
                    ActivityCompat.requestPermissions(Identify.this, new String[]{Manifest.permission.CAMERA}, TAKE_PHOTO);
                } else {
                    // 启动相机程序
                    startCamera();
                }
            }
        });
    }

    private void startCamera() {
        // 创建一个File对象，用于保存摄像头拍下的图片，这里把图片命名为output_image.jpg
        // 并将它存放在手机SD卡的应用关联缓存目录下
        File outputImage = new File(getExternalCacheDir(), "output_image.jpg");
        // 对照片的更换设置
        try {
            // 如果上一次的照片存在，就删除
            if (outputImage.exists()) {
                outputImage.delete();
            }
            // 创建一个新的文件
            outputImage.createNewFile();
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 如果Android版本大于等于7.0

        if (Build.VERSION.SDK_INT >= 24) {
            // 将File对象转换成一个封装过的Uri对象
            ImageUri = FileProvider.getUriForFile(this, "com.example.duonachi.camera.fileprovider", outputImage);
            Log.d("MainActivity", outputImage.toString() + "手机系统版本高于Android7.0");
        } else {
            // 将File对象转换为Uri对象，这个Uri标识着output_image.jpg这张图片的本地真实路径
            Log.d("MainActivity", outputImage.toString() + "手机系统版本低于Android7.0");
            ImageUri = Uri.fromFile(outputImage);
        }
        // 动态申请权限
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions( this, new String[]{Manifest.permission.CAMERA}, TAKE_PHOTO);
        } else {
            // 启动相机程序
            Intent intent4 = new Intent("android.media.action.IMAGE_CAPTURE");
            // 指定图片的输出地址为imageUri
            intent4.putExtra(MediaStore.EXTRA_OUTPUT, ImageUri);
            startActivityForResult(intent4, TAKE_PHOTO);
        }
    }


    //重写OnActivity方法
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        //处理相册返回
        if (requestCode == REQUEST_CODE_SELECT_PHOTO && resultCode == RESULT_OK) {
            if (data != null) {
                ImageUri = data.getData();
                try {
                    // 将选定的图片解码为Bitmap
                    Bitmap bitmap = BitmapFactory.decodeStream(getContentResolver().openInputStream(ImageUri));
                    figure_ready_imageview.setImageBitmap(bitmap);

                    // 获取临时目录和创建唯一的文件名
                    File cacheDir = getCacheDir();
                    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss", Locale.US);
                    String fileName = "temp_image_" + dateFormat.format(new Date()) + ".jpg";
                    File tempFile = new File(cacheDir, fileName);

                    // 将Bitmap写入临时文件
                    try (FileOutputStream out = new FileOutputStream(tempFile)) {
                        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, out); // 使用JPEG格式，100表示不压缩
                        tempFilePath = tempFile.getAbsolutePath();
                        // 图片已成功保存到临时文件
                        //Toast.makeText(this, "Image saved to cache: " + tempFile.getAbsolutePath(), Toast.LENGTH_SHORT).show();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                }
            }
        }
        //处理相机返回
        if (requestCode == TAKE_PHOTO && resultCode == RESULT_OK ) {
            try {
                // 将图片解析成Bitmap对象
                Bitmap bitmap = BitmapFactory.decodeStream(getContentResolver().openInputStream(ImageUri));
                figure_ready_imageview.setImageBitmap(bitmap);

                // 获取临时目录和创建唯一的文件名
                File cacheDir = getCacheDir();
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss", Locale.US);
                String fileName = "temp_image_" + dateFormat.format(new Date()) + ".jpg";
                File tempFile = new File(cacheDir, fileName);

                // 将Bitmap写入临时文件
                try (FileOutputStream out = new FileOutputStream(tempFile)) {
                    bitmap.compress(Bitmap.CompressFormat.JPEG, 100, out); // 使用JPEG格式，100表示不压缩
                    tempFilePath = tempFile.getAbsolutePath();
                    // 图片已成功保存到临时文件
                    //Toast.makeText(this, "Image saved to cache: " + tempFile.getAbsolutePath(), Toast.LENGTH_SHORT).show();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }
    }
}